@@ -14,16 +14,16 @@ extern "C" {
 #endif
 
 int grib_f_read_any_headers_only_from_file_(int *fid, char *buffer, size_t *nbytes);
-int grib_f_read_any_headers_only_from_file__(int *fid, char *buffer, size_t *nbytes);
-int grib_f_read_any_headers_only_from_file(int *fid, char *buffer, size_t *nbytes);
-int grib_f_read_any_from_file_(int *fid, char *buffer, size_t *nbytes);
-int grib_f_read_any_from_file__(int *fid, char *buffer, size_t *nbytes);
-int grib_f_read_any_from_file(int *fid, char *buffer, size_t *nbytes);
-int grib_f_write_file_(int *fid, char *buffer, size_t *nbytes);
-int grib_f_write_file__(int *fid, char *buffer, size_t *nbytes);
-int grib_f_write_file(int *fid, char *buffer, size_t *nbytes);
-int grib_f_read_file_(int *fid, char *buffer, size_t *nbytes);
-int grib_f_read_file__(int *fid, char *buffer, size_t *nbytes);
+int grib_f_write_file_(int* fid, void* buffer, size_t* nbytes);
+int any_f_new_from_loaded_(int* msgid, int* gid);
+int codes_f_bufr_copy_data_(int* gid1,int* gid2);
+int grib_f_get_string_array_(int* gid, char* key, char* val,int* nvals,int* slen, int len);
+int codes_f_bufr_keys_iterator_rewind_(int* kiter);
+int any_f_scan_file_(int* fid, int* n);
+int grib_f_new_from_message_int_(int* gid, int* buffer , size_t* bufsize);
+int grib_f_copy_key_(int* gidsrc, char* key, int* giddest, int len);
+int grib_f_set_samples_path_(char* path, int len);
+
 int grib_f_read_file(int *fid, char *buffer, size_t *nbytes);
 int grib_f_open_file_(int *fid, char *name, char *op, int lname, int lop);
 int grib_f_open_file__(int *fid, char *name, char *op, int lname, int lop);
@@ -49,7 +49,6 @@ int grib_f_iterator_next(int *iterid, double *lat, double *lon, double *value);
 int grib_f_iterator_delete_(int *iterid);
 int grib_f_iterator_delete__(int *iterid);
 int grib_f_iterator_delete(int *iterid);
-
 int grib_f_keys_iterator_new_(int *gid, int *iterid, char *name_space, int len);
 int grib_f_keys_iterator_new__(int *gid, int *iterid, char *name_space, int len);
 int grib_f_keys_iterator_new(int *gid, int *iterid, char *name_space, int len);
@@ -59,24 +58,6 @@ int grib_f_keys_iterator_next(int *iterid);
 int grib_f_keys_iterator_delete_(int *iterid);
 int grib_f_keys_iterator_delete__(int *iterid);
 int grib_f_keys_iterator_delete(int *iterid);
-
-/*BUFR keys iterator*/
-int codes_f_bufr_keys_iterator_new_(int *gid, int *iterid);
-int codes_f_bufr_keys_iterator_new__(int *gid, int *iterid);
-int codes_f_bufr_keys_iterator_new(int *gid, int *iterid);
-int codes_f_bufr_keys_iterator_next_(int *iterid);
-int codes_f_bufr_keys_iterator_next__(int *iterid);
-int codes_f_bufr_keys_iterator_next(int *iterid);
-int codes_f_bufr_keys_iterator_get_name_(int *iterid, char *name, int len);
-int codes_f_bufr_keys_iterator_get_name__(int *kiter, char *name, int len);
-int codes_f_bufr_keys_iterator_get_name(int *kiter, char *name, int len);
-int codes_f_bufr_keys_iterator_rewind_(int *kiter);
-int codes_f_bufr_keys_iterator_rewind__(int *kiter);
-int codes_f_bufr_keys_iterator_rewind(int *kiter);
-int codes_f_bufr_keys_iterator_delete_(int *iterid);
-int codes_f_bufr_keys_iterator_delete__(int *iterid);
-int codes_f_bufr_keys_iterator_delete(int *iterid);
-
 int grib_f_gribex_mode_on_(void);
 int grib_f_gribex_mode_on__(void);
 int grib_f_gribex_mode_on(void);
@@ -116,9 +97,9 @@ int grib_f_new_from_message_copy(int *gid, void *buffer, size_t *bufsize);
 int grib_f_new_from_samples_(int *gid, char *name, int lname);
 int grib_f_new_from_samples__(int *gid, char *name, int lname);
 int grib_f_new_from_samples(int *gid, char *name, int lname);
-int codes_bufr_f_new_from_samples_(int *gid, char *name, int lname);
-int codes_bufr_f_new_from_samples__(int *gid, char *name, int lname);
-int codes_bufr_f_new_from_samples(int *gid, char *name, int lname);
+int grib_f_new_from_template_(int *gid, char *name, int lname);
+int grib_f_new_from_template__(int *gid, char *name, int lname);
+int grib_f_new_from_template(int *gid, char *name, int lname);
 int grib_f_clone_(int *gidsrc, int *giddest);
 int grib_f_clone__(int *gidsrc, int *giddest);
 int grib_f_clone(int *gidsrc, int *giddest);
@@ -128,47 +109,12 @@ int grib_f_util_sections_copy(int *gidfrom, int *gidto, int *what, int *gidout);
 int grib_f_copy_namespace_(int *gidsrc, char *name, int *giddest, int len);
 int grib_f_copy_namespace__(int *gidsrc, char *name, int *giddest, int len);
 int grib_f_copy_namespace(int *gidsrc, char *name, int *giddest, int len);
-
-int grib_f_copy_key_ (int *gidsrc, char *name, int *giddest, int len);
-int grib_f_copy_key__(int *gidsrc, char *name, int *giddest, int len);
-int grib_f_copy_key  (int *gidsrc, char *name, int *giddest, int len);
-
 int grib_f_count_in_file(int *fid, int *n);
 int grib_f_count_in_file_(int *fid, int *n);
 int grib_f_count_in_file__(int *fid, int *n);
-
-int any_f_new_from_file_(int *fid, int *gid);
-int any_f_new_from_file__(int *fid, int *gid);
-int any_f_new_from_file(int *fid, int *gid);
-
-int any_f_scan_file_(int* fid,int* n);
-int any_f_scan_file__(int* fid,int* n);
-int any_f_scan_file(int* fid,int* n);
-
-int any_f_new_from_scanned_file_(int* fid,int* msgid,int* gid);
-int any_f_new_from_scanned_file__(int* fid,int* msgid,int* gid);
-int any_f_new_from_scanned_file(int* fid,int* msgid,int* gid);
-
-int any_f_load_all_from_file_(int* fid,int* n);
-int any_f_load_all_from_file__(int* fid,int* n);
-int any_f_load_all_from_file(int* fid,int* n);
-
-int any_f_new_from_loaded_(int* msgid,int* gid);
-int any_f_new_from_loaded__(int* msgid,int* gid);
-int any_f_new_from_loaded(int* msgid,int* gid);
-
-int codes_f_clear_loaded_from_file_();
-int codes_f_clear_loaded_from_file__();
-int codes_f_clear_loaded_from_file();
-
 int grib_f_new_from_file_(int *fid, int *gid);
 int grib_f_new_from_file__(int *fid, int *gid);
 int grib_f_new_from_file(int *fid, int *gid);
-
-int bufr_f_new_from_file_(int *fid, int *gid);
-int bufr_f_new_from_file__(int *fid, int *gid);
-int bufr_f_new_from_file(int *fid, int *gid);
-
 int grib_f_headers_only_new_from_file_(int *fid, int *gid);
 int grib_f_headers_only_new_from_file__(int *fid, int *gid);
 int grib_f_headers_only_new_from_file(int *fid, int *gid);
@@ -263,11 +209,9 @@ int grib_f_set_missing_(int *gid, char *key, int len);
 int grib_f_set_missing__(int *gid, char *key, int len);
 int grib_f_set_missing(int *gid, char *key, int len);
 int grib_f_is_missing_(int *gid, char *key, int *isMissing, int len);
+int grib_f_is_defined_(int *gid, char *key, int *isDefined, int len);
 int grib_f_is_missing__(int *gid, char *key, int *isMissing, int len);
 int grib_f_is_missing(int *gid, char *key, int *isMissing, int len);
-int grib_f_is_defined_(int* gid, char* key,int* isDefined,int len);
-int grib_f_is_defined__(int* gid, char* key,int* isDefined,int len);
-int grib_f_is_defined(int* gid, char* key,int* isDefined,int len);
 int grib_f_set_real4_(int *gid, char *key, float *val, int len);
 int grib_f_set_real4__(int *gid, char *key, float *val, int len);
 int grib_f_set_real4(int *gid, char *key, float *val, int len);
@@ -331,14 +275,6 @@ int grib_f_set_real8_array(int *gid, char *key, double *val, int *size, int len)
 int grib_f_set_force_real8_array_(int *gid, char *key, double *val, int *size, int len);
 int grib_f_set_force_real8_array__(int *gid, char *key, double *val, int *size, int len);
 int grib_f_set_force_real8_array(int *gid, char *key, double *val, int *size, int len);
-
-int grib_f_get_string_array(int* gid, char* key, char* val,int* nvals,int* slen,int len);
-int grib_f_get_string_array_(int* gid, char* key, char* val,int* nvals,int* slen,int len);
-int grib_f_get_string_array__(int* gid, char* key, char* val,int* nvals,int* slen,int len);
-int grib_f_set_string_array(int* gid, char* key, char* val,int* nvals,int* slen, int len);
-int grib_f_set_string_array_(int* gid, char* key, char* val,int* nvals,int* slen, int len);
-int grib_f_set_string_array__(int* gid, char* key, char* val,int* nvals,int* slen, int len);
-
 int grib_f_get_string_(int *gid, char *key, char *val, int len, int len2);
 int grib_f_get_string__(int *gid, char *key, char *val, int len, int len2);
 int grib_f_get_string(int *gid, char *key, char *val, int len, int len2);
@@ -370,24 +306,26 @@ int grib_f_multi_append_(int *ingid, int *sec, int *mgid);
 int grib_f_multi_append(int *ingid, int *sec, int *mgid);
 int grib_f_multi_append__(int *ingid, int *sec, int *mgid);
 
-int codes_f_bufr_copy_data(int* gid1,int* gid2);
-int codes_f_bufr_copy_data_(int* gid1,int* gid2);
-int codes_f_bufr_copy_data__(int* gid1,int* gid2);
-
-int grib_f_set_definitions_path_(char *path, int len);
-int grib_f_set_definitions_path__(char *path, int len);
-int grib_f_set_definitions_path(char *path, int len);
-int grib_f_set_samples_path_(char *path, int len);
-int grib_f_set_samples_path__(char *path, int len);
-int grib_f_set_samples_path(char *path, int len);
-
-int grib_f_julian_to_datetime(double* jd,long* year,long* month,long* day,long *hour,long *minute,long *second);
+int codes_f_bufr_keys_iterator_new_(int* gid,int* iterid);
+int grib_f_read_file_(int* fid, void* buffer, size_t* nbytes);
+int codes_f_bufr_keys_iterator_delete_(int* iterid);
+int grib_f_set_definitions_path_(char* path, int len);
+int grib_f_read_any_from_file_(int* fid, void* buffer, size_t* nbytes);
+int any_f_new_from_file_(int* fid, int* gid);
+int grib_f_get_native_type_(int* gid, char* key, int* val, int len);
+int grib_f_get_api_version_(int* apiVersion,int len);
+int codes_bufr_f_new_from_samples_(int* gid, char* name, int lname);
+int any_f_load_all_from_file_(int* fid, int* n);
+int codes_f_bufr_multi_element_constant_arrays_on_();
+int bufr_f_new_from_file_(int* fid, int* gid);
 int grib_f_julian_to_datetime_(double* jd,long* year,long* month,long* day,long *hour,long *minute,long *second);
-int grib_f_julian_to_datetime__(double* jd,long* year,long* month,long* day,long *hour,long *minute,long *second);
-
-int grib_f_datetime_to_julian(long* year,long* month,long* day, long* hour,long* minute,long* second,double* jd);
+int any_f_new_from_scanned_file_(int* fid, int* msgid, int* gid);
+int codes_f_bufr_keys_iterator_next_(int* iterid);
 int grib_f_datetime_to_julian_(long* year,long* month,long* day, long* hour,long* minute,long* second,double* jd);
-int grib_f_datetime_to_julian__(long* year,long* month,long* day, long* hour,long* minute,long* second,double* jd);
+int grib_f_set_string_array_(int* gid, char* key, char* val,int* nvals,int* slen, int len);
+int codes_f_bufr_keys_iterator_get_name_(int* iterid, char* name, int len);
+int codes_f_bufr_multi_element_constant_arrays_off_();
+
 
 #ifdef __cplusplus
 }
