@@ -8,13 +8,8 @@
  * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
-/**************************************
- *  Enrico Fucile
- **************************************/
-
-
 #include "grib_api_internal.h"
-#include <ctype.h>
+#include <cctype>
 /*
    This is used by make_class.pl
 
@@ -90,7 +85,6 @@ grib_dumper_class* grib_dumper_class_wmo = &_grib_dumper_class_wmo;
 
 /* END_CLASS_IMP */
 static void set_begin_end(grib_dumper* d, grib_accessor* a);
-static void print_offset(FILE* out, long begin, long theEnd);
 static void print_hexadecimal(FILE* out, unsigned long flags, grib_accessor* a);
 
 static void init_class(grib_dumper_class* c) {}
@@ -108,6 +102,18 @@ static int destroy(grib_dumper* d)
     return GRIB_SUCCESS;
 }
 
+static void print_offset(FILE* out, long begin, long theEnd, int width=10)
+{
+    char tmp[50];
+
+    if (begin == theEnd)
+        fprintf(out, "%-*ld", width, begin);
+    else {
+        snprintf(tmp, sizeof(tmp), "%ld-%ld", begin, theEnd);
+        fprintf(out, "%-*s", width, tmp);
+    }
+}
+
 static void aliases(grib_dumper* d, grib_accessor* a)
 {
     int i;
@@ -146,15 +152,15 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
         (d->option_flags & GRIB_DUMP_FLAG_CODED) != 0)
         return;
 
-    grib_value_count(a, &count);
+    a->value_count(&count);
     size = count;
 
     if (size > 1) {
         values = (long*)grib_context_malloc_clear(a->context, sizeof(long) * size);
-        err    = grib_unpack_long(a, values, &size);
+        err    = a->unpack_long(values, &size);
     }
     else {
-        err = grib_unpack_long(a, &value, &size);
+        err = a->unpack_long(&value, &size);
     }
 
     if ((a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0 &&
@@ -186,7 +192,7 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
         }
     }
     else {
-        if (((a->flags & GRIB_ACCESSOR_FLAG_CAN_BE_MISSING) != 0) && grib_is_missing_internal(a))
+        if (((a->flags & GRIB_ACCESSOR_FLAG_CAN_BE_MISSING) != 0) && a->is_missing_internal())
             fprintf(self->dumper.out, "%s = MISSING", a->name);
         else
             fprintf(self->dumper.out, "%s = %ld", a->name, value);
@@ -222,10 +228,10 @@ static void dump_bits(grib_dumper* d, grib_accessor* a, const char* comment)
         (d->option_flags & GRIB_DUMP_FLAG_CODED) != 0)
         return;
 
-    err = grib_unpack_long(a, &value, &size);
+    err = a->unpack_long(&value, &size);
     set_begin_end(d, a);
 
-    /*for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");*/
+    //for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");
     print_offset(self->dumper.out, self->begin, self->theEnd);
     if ((d->option_flags & GRIB_DUMP_FLAG_TYPE) != 0)
         fprintf(self->dumper.out, "%s (int) ", a->creator->op);
@@ -239,7 +245,7 @@ static void dump_bits(grib_dumper* d, grib_accessor* a, const char* comment)
     }
 
     if(comment) {
-        /* ECC-1186: Whole comment is too big, so pick the part that follows the ':' i.e. flag table file */
+        // ECC-1186: Whole comment is too big, so pick the part that follows the ':' i.e. flag table file
         const char* p = strchr(comment, ':');
         if (p) fprintf(self->dumper.out," (%s) ]", p+1);
         else   fprintf(self->dumper.out, "]");
@@ -268,20 +274,20 @@ static void dump_double(grib_dumper* d, grib_accessor* a, const char* comment)
         (d->option_flags & GRIB_DUMP_FLAG_CODED) != 0)
         return;
 
-    err = grib_unpack_double(a, &value, &size);
+    err = a->unpack_double(&value, &size);
     set_begin_end(d, a);
 
-    /*for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");*/
+    //for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");
 
     print_offset(self->dumper.out, self->begin, self->theEnd);
     if ((d->option_flags & GRIB_DUMP_FLAG_TYPE) != 0)
         fprintf(self->dumper.out, "%s (double) ", a->creator->op);
 
-    if (((a->flags & GRIB_ACCESSOR_FLAG_CAN_BE_MISSING) != 0) && grib_is_missing_internal(a))
+    if (((a->flags & GRIB_ACCESSOR_FLAG_CAN_BE_MISSING) != 0) && a->is_missing_internal())
         fprintf(self->dumper.out, "%s = MISSING", a->name);
     else
         fprintf(self->dumper.out, "%s = %g", a->name, value);
-    /*if(comment) fprintf(self->dumper.out," [%s]",comment);*/
+    // if(comment) fprintf(self->dumper.out," [%s]",comment);
 
     if (err == 0)
         print_hexadecimal(self->dumper.out, d->option_flags, a);
@@ -305,13 +311,13 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
         return;
     }
 
-    ecc__grib_get_string_length(a, &size);
+    grib_get_string_length_acc(a, &size);
     value = (char*)grib_context_malloc_clear(a->context, size);
     if (!value) {
         grib_context_log(a->context, GRIB_LOG_ERROR, "unable to allocate %zu bytes", size);
         return;
     }
-    err = grib_unpack_string(a, value, &size);
+    err = a->unpack_string(value, &size);
     p   = value;
 
     set_begin_end(d, a);
@@ -322,7 +328,7 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
         p++;
     }
 
-    /*for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");*/
+    //for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");
     print_offset(self->dumper.out, self->begin, self->theEnd);
     if ((d->option_flags & GRIB_DUMP_FLAG_TYPE) != 0)
         fprintf(self->dumper.out, "%s (str) ", a->creator->op);
@@ -332,7 +338,7 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
     if (err == 0)
         print_hexadecimal(self->dumper.out, d->option_flags, a);
 
-    /*if(comment) fprintf(self->dumper.out," [%s]",comment);*/
+    // if(comment) fprintf(self->dumper.out," [%s]",comment);
     if (err)
         fprintf(self->dumper.out, " *** ERR=%d (%s) [grib_dumper_wmo::dump_string]", err, grib_get_error_message(err));
     aliases(d, a);
@@ -354,7 +360,7 @@ static void dump_bytes(grib_dumper* d, grib_accessor* a, const char* comment)
 
     set_begin_end(d, a);
 
-    /*for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");*/
+    // for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");
     print_offset(self->dumper.out, self->begin, self->theEnd);
     if ((d->option_flags & GRIB_DUMP_FLAG_TYPE) != 0)
         fprintf(self->dumper.out, "%s ", a->creator->op);
@@ -375,7 +381,7 @@ static void dump_bytes(grib_dumper* d, grib_accessor* a, const char* comment)
 
     fprintf(self->dumper.out, "\n");
 
-    err = grib_unpack_bytes(a, buf, &size);
+    err = a->unpack_bytes(buf, &size);
     if (err) {
         grib_context_free(d->context, buf);
         fprintf(self->dumper.out, " *** ERR=%d (%s) [grib_dumper_wmo::dump_bytes]\n}", err, grib_get_error_message(err));
@@ -388,7 +394,7 @@ static void dump_bytes(grib_dumper* d, grib_accessor* a, const char* comment)
     }
 
     k = 0;
-    /* if(size > 100) size = 100;  */
+    // if(size > 100) size = 100;
     while (k < size) {
         int j;
         for (i = 0; i < d->depth + 3; i++)
@@ -427,7 +433,7 @@ static void dump_values(grib_dumper* d, grib_accessor* a)
         (d->option_flags & GRIB_DUMP_FLAG_CODED) != 0)
         return;
 
-    grib_value_count(a, &count);
+    a->value_count(&count);
     size = count;
 
     if (size == 1) {
@@ -438,16 +444,15 @@ static void dump_values(grib_dumper* d, grib_accessor* a)
 
     set_begin_end(d, a);
 
-    /* For the DIAG pseudo GRIBs. Key charValues uses 1-byte integers to represent a character */
+    // For the DIAG pseudo GRIBs. Key charValues uses 1-byte integers to represent a character
     if (a->flags & GRIB_ACCESSOR_FLAG_STRING_TYPE) {
         is_char = 1;
     }
 
-    /*for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");*/
-    print_offset(self->dumper.out, self->begin, self->theEnd);
+    print_offset(self->dumper.out, self->begin, self->theEnd, 12); // ECC-1749
     if ((d->option_flags & GRIB_DUMP_FLAG_TYPE) != 0) {
         char type_name[32]     = "";
-        const long native_type = grib_accessor_get_native_type(a);
+        const long native_type = a->get_native_type();
         if (native_type == GRIB_TYPE_LONG)
             strcpy(type_name, "(int)");
         else if (native_type == GRIB_TYPE_DOUBLE)
@@ -471,7 +476,7 @@ static void dump_values(grib_dumper* d, grib_accessor* a)
 
     fprintf(self->dumper.out, "\n");
 
-    err = grib_unpack_double(a, buf, &size);
+    err = a->unpack_double(buf, &size);
 
     if (err) {
         grib_context_free(d->context, buf);
@@ -486,9 +491,7 @@ static void dump_values(grib_dumper* d, grib_accessor* a)
 
     k = 0;
     while (k < size) {
-#if 1
         int j;
-        /*for(i = 0; i < d->depth + 3 ; i++) fprintf(self->dumper.out," ");*/
         for (j = 0; j < 8 && k < size; j++, k++) {
             if (is_char)
                 fprintf(self->dumper.out, "'%c'", (char)buf[k]);
@@ -498,31 +501,25 @@ static void dump_values(grib_dumper* d, grib_accessor* a)
                 fprintf(self->dumper.out, ", ");
         }
         fprintf(self->dumper.out, "\n");
-#else
-
-        if (is_char)
-            fprintf(self->dumper.out, "%d '%c'\n", k, (char)buf[k]);
-        else
-            fprintf(self->dumper.out, "%d %g\n", k, buf[k]);
-
-#endif
+        // if (is_char)
+        //     fprintf(self->dumper.out, "%d '%c'\n", k, (char)buf[k]);
+        // else
+        //     fprintf(self->dumper.out, "%d %g\n", k, buf[k]);
     }
     if (more) {
-        /*for(i = 0; i < d->depth + 3 ; i++) fprintf(self->dumper.out," ");*/
+        //for(i = 0; i < d->depth + 3 ; i++) fprintf(self->dumper.out," ");
         fprintf(self->dumper.out, "... %lu more values\n", (unsigned long)more);
     }
 
-    /*for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");*/
     fprintf(self->dumper.out, "} # %s %s \n", a->creator->op, a->name);
     grib_context_free(d->context, buf);
 }
 
 static void dump_label(grib_dumper* d, grib_accessor* a, const char* comment)
 {
-    /*grib_dumper_wmo *self = (grib_dumper_wmo*)d;
-
-  for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");
-  fprintf(self->dumper.out,"----> %s %s %s\n",a->creator->op, a->name,comment?comment:"");*/
+//  grib_dumper_wmo *self = (grib_dumper_wmo*)d;
+//  for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");
+//  fprintf(self->dumper.out,"----> %s %s %s\n",a->creator->op, a->name,comment?comment:"");
 }
 
 static void dump_section(grib_dumper* d, grib_accessor* a, grib_block_of_accessors* block)
@@ -536,7 +533,6 @@ static void dump_section(grib_dumper* d, grib_accessor* a, grib_block_of_accesso
     if (!strncmp(a->name, "section", 7))
         is_wmo_section = 1;
 
-    /*for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");*/
     if (is_wmo_section) {
         upper = (char*)malloc(strlen(a->name) + 1);
         Assert(upper);
@@ -556,13 +552,13 @@ static void dump_section(grib_dumper* d, grib_accessor* a, grib_block_of_accesso
     else {
     }
 
-    /*printf("------------- section_offset = %ld\n",self->section_offset);*/
+    //printf("------------- section_offset = %ld\n",self->section_offset);
     d->depth += 3;
     grib_dump_accessors_block(d, block);
     d->depth -= 3;
 
-    /*for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");*/
-    /*fprintf(self->dumper.out,"<===== %s %s\n",a->creator->op, a->name);*/
+    //for(i = 0; i < d->depth ; i++) fprintf(self->dumper.out," ");
+    //fprintf(self->dumper.out,"<===== %s %s\n",a->creator->op, a->name);
 }
 
 static void set_begin_end(grib_dumper* d, grib_accessor* a)
@@ -570,22 +566,11 @@ static void set_begin_end(grib_dumper* d, grib_accessor* a)
     grib_dumper_wmo* self = (grib_dumper_wmo*)d;
     if ((d->option_flags & GRIB_DUMP_FLAG_OCTET) != 0) {
         self->begin  = a->offset - self->section_offset + 1;
-        self->theEnd = grib_get_next_position_offset(a) - self->section_offset;
+        self->theEnd = a->get_next_position_offset() - self->section_offset;
     }
     else {
         self->begin  = a->offset;
-        self->theEnd = grib_get_next_position_offset(a);
-    }
-}
-
-static void print_offset(FILE* out, long begin, long theEnd)
-{
-    char tmp[50];
-    if (begin == theEnd)
-        fprintf(out, "%-10ld", begin);
-    else {
-        snprintf(tmp, sizeof(tmp), "%ld-%ld", begin, theEnd);
-        fprintf(out, "%-10s", tmp);
+        self->theEnd = a->get_next_position_offset();
     }
 }
 
@@ -619,7 +604,7 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
         return;
 
     c = a->context;
-    grib_value_count(a, &count);
+    a->value_count(&count);
     if (count == 0)
         return;
     size = count;
@@ -634,9 +619,9 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
         return;
     }
 
-    err = grib_unpack_string_array(a, values, &size);
+    err = a->unpack_string_array(values, &size);
 
-    /* print_offset(self->dumper.out,d,a); */
+    // print_offset(self->dumper.out,d,a);
     print_offset(self->dumper.out, self->begin, self->theEnd);
 
     if ((d->option_flags & GRIB_DUMP_FLAG_TYPE) != 0) {
