@@ -9,7 +9,7 @@
  */
 
 #include "grib_api_internal.h"
-#include <math.h>
+#include <cmath>
 
 /*
    This is used by make_class.pl
@@ -81,6 +81,8 @@ static void init_class(grib_iterator_class* c)
 }
 /* END_CLASS_IMP */
 
+#define ITER "Lambert azimuthal equal area Geoiterator"
+
 static int next(grib_iterator* iter, double* lat, double* lon, double* val)
 {
     grib_iterator_lambert_azimuthal_equal_area* self = (grib_iterator_lambert_azimuthal_equal_area*)iter;
@@ -207,7 +209,11 @@ static int init_oblate(grib_handle* h,
     sinphi_  = sin(standardParallelInRadians); /*  (P->phi0); */
     Q__sinb1 = pj_qsfn(sinphi_, e, one_es) / Q__qp;
     Q__cosb1 = sqrt(1.0 - Q__sinb1 * Q__sinb1);
-    Q__dd    = cos(standardParallelInRadians) / (sqrt(1. - es * sinphi_ * sinphi_) * Q__rq * Q__cosb1);
+    if (Q__cosb1 == 0) {
+        Q__dd = 1.0;
+    } else {
+        Q__dd = cos(standardParallelInRadians) / (sqrt(1. - es * sinphi_ * sinphi_) * Q__rq * Q__cosb1);
+    }
     Q__ymf   = (Q__xmf = Q__rq) / Q__dd;
     Q__xmf *= Q__dd;
 
@@ -227,12 +233,12 @@ static int init_oblate(grib_handle* h,
     /* Allocate latitude and longitude arrays */
     self->lats = (double*)grib_context_malloc(h->context, nv * sizeof(double));
     if (!self->lats) {
-        grib_context_log(h->context, GRIB_LOG_ERROR, "Error allocating %ld bytes", nv * sizeof(double));
+        grib_context_log(h->context, GRIB_LOG_ERROR, "%s: Error allocating %zu bytes", ITER, nv * sizeof(double));
         return GRIB_OUT_OF_MEMORY;
     }
     self->lons = (double*)grib_context_malloc(h->context, nv * sizeof(double));
     if (!self->lats) {
-        grib_context_log(h->context, GRIB_LOG_ERROR, "Error allocating %ld bytes", nv * sizeof(double));
+        grib_context_log(h->context, GRIB_LOG_ERROR, "%s: Error allocating %zu bytes", ITER, nv * sizeof(double));
         return GRIB_OUT_OF_MEMORY;
     }
     lats = self->lats;
@@ -251,7 +257,12 @@ static int init_oblate(grib_handle* h,
                 xy_y *= Q__dd;
                 rho = hypot(xy_x, xy_y);
                 Assert(rho >= EPS10); /* TODO(masn): check */
-                sCe = 2. * asin(.5 * rho / Q__rq);
+                const double asin_arg = (0.5 * rho / Q__rq);
+                if (asin_arg < -1.0 || asin_arg > 1.0) {
+                    grib_context_log(h->context, GRIB_LOG_ERROR, "Invalid value: arcsin argument=%g", asin_arg);
+                    return GRIB_GEOCALCULUS_PROBLEM;
+                }
+                sCe = 2. * asin(asin_arg);
                 cCe = cos(sCe);
                 sCe = sin(sCe);
                 xy_x *= sCe;
@@ -308,14 +319,12 @@ static int init_sphere(grib_handle* h,
     Dy         = jScansPositively == 1 ? Dy / 1000 : -Dy / 1000;
     self->lats = (double*)grib_context_malloc(h->context, nv * sizeof(double));
     if (!self->lats) {
-        grib_context_log(h->context, GRIB_LOG_ERROR,
-                         "Error allocating %ld bytes", nv * sizeof(double));
+        grib_context_log(h->context, GRIB_LOG_ERROR, "%s: Error allocating %zu bytes", ITER, nv * sizeof(double));
         return GRIB_OUT_OF_MEMORY;
     }
     self->lons = (double*)grib_context_malloc(h->context, nv * sizeof(double));
     if (!self->lats) {
-        grib_context_log(h->context, GRIB_LOG_ERROR,
-                         "Error allocating %ld bytes", nv * sizeof(double));
+        grib_context_log(h->context, GRIB_LOG_ERROR, "%s: Error allocating %zu bytes", ITER, nv * sizeof(double));
         return GRIB_OUT_OF_MEMORY;
     }
     lats = self->lats;
@@ -435,9 +444,7 @@ static int init(grib_iterator* iter, grib_handle* h, grib_arguments* args)
         return err;
 
     if (iter->nv != nx * ny) {
-        grib_context_log(h->context, GRIB_LOG_ERROR,
-                         "Wrong number of points (%ld!=%ldx%ld)",
-                         iter->nv, nx, ny);
+        grib_context_log(h->context, GRIB_LOG_ERROR, "%s: Wrong number of points (%zu!=%ldx%ld)", ITER, iter->nv, nx, ny);
         return GRIB_WRONG_GRID;
     }
     if ((err = grib_get_double_internal(h, slatFirstInDegrees, &latFirstInDegrees)) != GRIB_SUCCESS)
