@@ -36,6 +36,7 @@ if command -v "ncdump" >/dev/null 2>&1; then
     NC_DUMPER="ncdump"
 fi
 
+
 echo "Test ECC-1041: One parameter with different expvers ..."
 # ------------------------------------------------------------
 # This has 5 messages, all 'tp'. Change the first message to have a different expver
@@ -48,23 +49,22 @@ if test "x$NC_DUMPER" != "x"; then
     grep -q "short tp_0001" $tempText
 fi
 
-if [ $ECCODES_ON_WINDOWS -eq 0 ]; then
-    echo "Test HDF5 decoding ..."
-    # ---------------------------
-    # Note: this is only available in NetCDF-4. So need to check if the command worked with -k3
-    input=${data_dir}/sample.grib2
-    set +e
-    ${tools_dir}/grib_to_netcdf -k3 -o $tempNetcdf $input 2>/dev/null
-    stat=$?
-    set -e
-    if [ $stat -eq 0 ]; then
-        have_netcdf4=1
-        ${tools_dir}/grib_dump -TA -O $tempNetcdf
-        res=`${tools_dir}/grib_get -TA -p identifier $tempNetcdf`
-        [ "$res" = "HDF5" ]
-    fi
+echo "Test HDF5 decoding ..."
+# ---------------------------
+# Note: this is only available in NetCDF-4. So need to check if the command worked with -k3
+input=${data_dir}/sample.grib2
+set +e
+${tools_dir}/grib_to_netcdf -k3 -o $tempNetcdf $input 2>/dev/null
+stat=$?
+set -e
+if [ $stat -eq 0 ]; then
+    have_netcdf4=1
+    ${tools_dir}/grib_dump -TA -O $tempNetcdf
+    res=`${tools_dir}/grib_get -TA -p identifier $tempNetcdf`
+    [ "$res" = "HDF5" ]
 fi
 
+
 grib_files="\
  regular_latlon_surface.grib2 \
  regular_latlon_surface.grib1 \
@@ -110,6 +110,14 @@ ${tools_dir}/grib_to_netcdf -o $tempNetcdf $tempGrib
 ${tools_dir}/grib_set -s productDefinitionTemplateNumber=31 $sample2 $tempGrib
 ${tools_dir}/grib_to_netcdf -o $tempNetcdf $tempGrib
 
+ECCODES_DEBUG=-1 ${tools_dir}/grib_to_netcdf -o $tempNetcdf $tempGrib
+
+
+# The -u option
+input=${data_dir}/sample.grib2
+${tools_dir}/grib_to_netcdf -u time -o $tempNetcdf $input
+
+
 echo "Test different resolutions ..."
 # ------------------------------------
 # This should fail as messages have different resolutions
@@ -148,6 +156,49 @@ set -e
 [ $status -ne 0 ]
 grep -q "Wrong number of fields" $tempText
 
+
+echo "Not a regular grid ..."
+# --------------------------
+input=${data_dir}/reduced_gaussian_pressure_level.grib2
+set +e
+${tools_dir}/grib_to_netcdf -o $tempNetcdf $input > $tempText 2>&1
+status=$?
+set -e
+[ $status -ne 0 ]
+grep -q "not on a regular lat/lon grid or on a regular Gaussian grid" $tempText
+
+
+# ECC-1783: No error message when input file has invalid fields
+input=$data_dir/bad.grib
+set +e
+${tools_dir}/grib_to_netcdf -o $tempNetcdf $input > $tempText 2>&1
+status=$?
+set -e
+[ $status -ne 0 ]
+grep -q "Wrong message length" $tempText
+
+
+# Non-GRIB input
+input=$data_dir/bufr/aaen_55.bufr
+set +e
+${tools_dir}/grib_to_netcdf -o $tempNetcdf $input > $tempText 2>&1
+status=$?
+set -e
+[ $status -ne 0 ]
+grep -q "Input does not contain any field" $tempText
+
+
+# Bad reference date
+input=$data_dir/sample.grib2
+set +e
+${tools_dir}/grib_to_netcdf -Rxxx -o $tempNetcdf $input > $tempText 2>&1
+status=$?
+set -e
+[ $status -ne 0 ]
+grep -q "Invalid reference date" $tempText
+
+
+# Validity time check
 export GRIB_TO_NETCDF_CHECKVALIDTIME=0
 ${tools_dir}/grib_to_netcdf -o $tempNetcdf $tempGrib
 [ -f "$tempNetcdf" ]
