@@ -9,6 +9,7 @@
  */
 
 #include "grib_api_internal.h"
+#include "eccodes.h"
 
 #define NUMBER(x) (sizeof(x) / sizeof(x[0]))
 
@@ -40,25 +41,39 @@ static void check_float_representation(const double val, const double expected,
     compare_doubles(out, expected, tolerance);
 }
 
+static void test_get_package_name()
+{
+    printf("Running %s ...\n", __func__);
+
+    const char* pn = codes_get_package_name();
+    Assert(pn != NULL);
+    Assert(strlen(pn) > 1);
+    printf("Package name = %s\n", pn);
+}
+
 static void test_get_git_sha1()
 {
-    const char* sha1 = grib_get_git_sha1();
+    printf("Running %s ...\n", __func__);
+
+    const char* sha1 = codes_get_git_sha1();
     Assert(sha1 != NULL);
-    printf("Testing: test_get_git_sha1... %s\n", sha1);
+    printf("Git SHA1 = %s\n", sha1);
 }
 
 static void test_get_build_date()
 {
+    printf("Running %s ...\n", __func__);
+
     const char* bdate = codes_get_build_date();
     Assert(bdate != NULL);
     /* Should be of the format YYYY.MM.DD or empty (not implemented) */
     Assert( strlen(bdate) == 0 || isdigit(bdate[0]) );
-    printf("Testing: test_get_build_date... %s\n", bdate);
 }
 
 static void test_grib_nearest_smaller_ibmfloat()
 {
-    printf("Testing: test_grib_nearest_smaller_ibmfloat...\n");
+    printf("Running %s ...\n", __func__);
+
     check_float_representation(-1.0, -1.0, IBM_FLOAT);
     check_float_representation(0.0, 0.0, IBM_FLOAT);
     check_float_representation(1.0, 1.0, IBM_FLOAT);
@@ -69,7 +84,8 @@ static void test_grib_nearest_smaller_ibmfloat()
 
 static void test_grib_nearest_smaller_ieeefloat()
 {
-    printf("Testing: test_grib_nearest_smaller_ieeefloat...\n");
+    printf("Running %s ...\n", __func__);
+
     check_float_representation(-1.0, -1.0, IEEE_FLOAT);
     check_float_representation(0.0, 0.0, IEEE_FLOAT);
     check_float_representation(1.0, 1.0, IEEE_FLOAT);
@@ -80,12 +96,14 @@ static void test_grib_nearest_smaller_ieeefloat()
 
 static void test_gaussian_latitudes(int order)
 {
+    printf("Running %s ...\n", __func__);
+
     int ret       = 0;
     const int num = 2 * order;
     double lat1 = 0, lat2 = 0;
     double* lats = (double*)malloc(sizeof(double) * num);
-    printf("Testing: test_gaussian_latitudes order=%d...\n", order);
-    ret = grib_get_gaussian_latitudes(order, lats);
+
+    ret = codes_get_gaussian_latitudes(order, lats);
     Assert(ret == GRIB_SUCCESS);
 
     lat1 = lats[0];
@@ -98,15 +116,16 @@ static void test_gaussian_latitudes(int order)
 
 static void test_gaussian_latitude_640()
 {
-    /* Test all latitudes for one specific Gaussian number */
+    printf("Running %s ...\n", __func__);
+
+    /* Test latitudes for one specific Gaussian number */
     const int order        = 640;
     const int num          = 2 * order;
     int ret                = 0;
     const double tolerance = 1e-6;
     double* lats           = (double*)malloc(sizeof(double) * num);
-    ret                    = grib_get_gaussian_latitudes(order, lats);
+    ret                    = codes_get_gaussian_latitudes(order, lats);
     Assert(ret == GRIB_SUCCESS);
-    printf("Testing: test_gaussian_latitude_640...\n");
 
     compare_doubles(lats[0], 89.892396, tolerance);
     compare_doubles(lats[1], 89.753005, tolerance);
@@ -149,10 +168,11 @@ static void test_gaussian_latitude_640()
 
 static void test_string_splitting()
 {
+    printf("Running %s ...\n", __func__);
+
     int i          = 0;
     char input[80] = "Born|To|Be|Wild";
     char** list    = 0;
-    printf("Testing: test_string_splitting...\n");
 
     list           = string_split(input, "|");
     if (!list) { Assert(!"List is NULL"); return; }
@@ -201,14 +221,14 @@ static void my_assertion_proc(const char* message)
 
 static void test_assertion_catching()
 {
+    printf("Running %s ...\n", __func__);
+
     char empty[] = "";
     char** list  = 0;
     int i        = 0;
     Assert(assertion_caught == 0);
     codes_set_codes_assertion_failed_proc(&my_assertion_proc);
 
-    printf("Testing: test_assertion_catching...\n");
-
     /* Do something illegal */
     list = string_split(empty, " ");
 
@@ -223,7 +243,6 @@ static void test_assertion_catching()
     free(list);
 }
 
-
 static void my_logging_proc(const grib_context* c, int level, const char* mesg)
 {
     logging_caught = 1;
@@ -235,25 +254,25 @@ static void test_logging_proc()
 
     /* Override default behaviour */
     grib_context_set_logging_proc(context, my_logging_proc);
-    grib_context_log(context, GRIB_LOG_ERROR, "This error will be handled by me");
+    grib_context_log(context, GRIB_LOG_ERROR, "test_logging_proc: This error will be handled by me");
     Assert(logging_caught == 1);
 
     /* Restore the logging proc */
     logging_caught = 0;
     grib_context_set_logging_proc(context, NULL);
-    grib_context_log(context, GRIB_LOG_ERROR, "This will come out as normal");
+    grib_context_log(context, GRIB_LOG_ERROR, "test_logging_proc: This error will come out as normal");
     Assert(logging_caught == 0);
 }
 
 static void test_concept_condition_strings()
 {
+    printf("Running %s ...\n", __func__);
+
     int err           = 0;
     char result[1024] = {0,};
     grib_context* context = NULL;
     grib_handle* h = grib_handle_new_from_samples(context, "GRIB2");
 
-    printf("Testing: test_concept_condition_strings...\n");
-
     err = get_concept_condition_string(h, "typeOfLevel", NULL, result);
     Assert(!err);
     Assert(strcmp(result, "typeOfFirstFixedSurface=1,typeOfSecondFixedSurface=255") == 0);
@@ -273,8 +292,10 @@ static void test_concept_condition_strings()
     grib_handle_delete(h);
 }
 
-static void test_trimming()
+static void test_string_trimming()
 {
+    printf("Running %s ...\n", __func__);
+
     char a[] = " Standing  ";
     char b[] = "  Weeping ";
     char c[] = "  Silhouette ";
@@ -286,8 +307,6 @@ static void test_trimming()
     char* pD = d;
     char* pE = e;
 
-    printf("Testing: test_trimming...\n");
-
     string_lrtrim(&pA, 0, 1); /*right only*/
     Assert( strcmp(pA, " Standing")==0 );
 
@@ -306,7 +325,8 @@ static void test_trimming()
 
 static void test_string_ends_with()
 {
-    printf("Testing: test_string_ends_with...\n");
+    printf("Running %s ...\n", __func__);
+
     Assert( string_ends_with("GRIB2.tmpl", "tmpl") == 1 );
     Assert( string_ends_with("GRIB2.tmpl", ".tmpl") == 1 );
     Assert( string_ends_with("", "") == 1 );
@@ -319,28 +339,97 @@ static void test_string_ends_with()
     Assert( string_ends_with("GRIB2.tmpl", " ") == 0 );
 }
 
+static void test_string_to_long()
+{
+    printf("Running %s ...\n", __func__);
+
+    long lVal = 0;
+    Assert( string_to_long("0", &lVal, 1) == GRIB_SUCCESS);
+    Assert( lVal == 0 );
+
+    Assert( string_to_long("42", &lVal, 1) == GRIB_SUCCESS);
+    Assert( lVal == 42 );
+
+    Assert( string_to_long("-1", &lVal, 1) == GRIB_SUCCESS);
+    Assert( lVal == -1 );
+    Assert( string_to_long("+999", &lVal, 1) == GRIB_SUCCESS);
+    Assert( lVal == 999 );
+
+    Assert( string_to_long("15MB", &lVal, 0) == GRIB_SUCCESS);
+    Assert( lVal == 15 );
+
+    // illegal cases
+    Assert( string_to_long("4000000000000000000000", &lVal, 1) == GRIB_INVALID_ARGUMENT);
+    Assert( string_to_long("XY", &lVal, 1) == GRIB_INVALID_ARGUMENT);
+    Assert( string_to_long("A6", &lVal, 1) == GRIB_INVALID_ARGUMENT);
+    Assert( string_to_long("5K", &lVal, 1) == GRIB_INVALID_ARGUMENT);
+}
+
+static void test_string_replace_char()
+{
+    printf("Running %s ...\n", __func__);
+
+    char input[32] = {0,};
+    strncpy(input, "Mask Of Zoro", sizeof(input));
+    string_replace_char(input, ' ', '-');
+    Assert(STR_EQUAL(input, "Mask-Of-Zoro"));
+}
+
+static void test_string_remove_char()
+{
+    printf("Running %s ...\n", __func__);
+    char input[64] = "a:b:c";
+    string_remove_char(input, ':');
+    Assert(STR_EQUAL(input, "abc"));
+}
+
 static void test_gribex_mode()
 {
     grib_context* c = grib_context_get_default();
-    printf("Testing: test_gribex_mode...\n");
+    printf("Running %s ...\n", __func__);
 
     Assert( grib_get_gribex_mode(c) == 0 ); /* default is OFF */
     grib_gribex_mode_on(c);
+    codes_gribex_mode_on(c);
     Assert( grib_get_gribex_mode(c) == 1 );
     grib_gribex_mode_off(c);
+    codes_gribex_mode_off(c);
     Assert( grib_get_gribex_mode(c) == 0 );
+    Assert( codes_get_gribex_mode(c) == 0 );
+}
+
+static void test_gts_header_mode()
+{
+    grib_context* c = grib_context_get_default();
+    printf("Running %s ...\n", __func__);
+
+    grib_gts_header_on(c);
+    codes_gts_header_on(c);
+    Assert(c->gts_header_on == 1);
+    grib_gts_header_off(c);
+    codes_gts_header_off(c);
+    Assert(c->gts_header_on == 0);
+}
+
+static void test_bufr_multi_element_constant_arrays()
+{
+    grib_context* c = grib_context_get_default();
+    printf("Running %s ...\n", __func__);
+
+    codes_bufr_multi_element_constant_arrays_on(c);
+    codes_bufr_multi_element_constant_arrays_off(c);
 }
 
 static void test_grib_binary_search()
 {
+    printf("Running %s ...\n", __func__);
+
     double array_asc[] = {-0.1, 33.4, 56.1, 101.8};
     double array_desc[] = {88, 78, 0, -88};
     const size_t idx_asc_max = NUMBER(array_asc) - 1;
     const size_t idx_desc_max = NUMBER(array_desc) - 1;
     size_t idx_upper=0, idx_lower = 0;
 
-    printf("Testing: test_grib_binary_search...\n");
-
     grib_binary_search(array_asc, idx_asc_max, 56.0, &idx_upper, &idx_lower);
     Assert(idx_lower == 1 && idx_upper == 2);
     grib_binary_search(array_asc, idx_asc_max, 56.1, &idx_upper, &idx_lower);
@@ -358,6 +447,8 @@ static void test_grib_binary_search()
 
 static void test_parse_keyval_string()
 {
+    printf("Running %s ...\n", __func__);
+
     int err = 0;
     int values_required = 1;
     int count = 0;
@@ -369,13 +460,13 @@ static void test_parse_keyval_string()
     char input2[] = "x=14";
     char input3[] = "mars.level=0.978";
 
-    printf("Testing: parse_keyval_string...\n");
-
     count = max_count;
     err = parse_keyval_string(NULL, input1,
                               values_required, GRIB_TYPE_UNDEFINED, values1, &count);
     Assert( !err );
     Assert( count == 2 );
+    grib_print_values("print values test: values1", values1, stdout);
+
     Assert( strcmp(values1[0].name, "key1")==0 );
     Assert( strcmp(values1[0].string_value, "value1")==0 );
     Assert( values1[0].equal == 1 );
@@ -394,6 +485,7 @@ static void test_parse_keyval_string()
                               values_required, GRIB_TYPE_LONG, values2, &count);
     Assert( !err );
     Assert( count == 1 );
+    grib_print_values("print values test: values2", values2, stdout);
     Assert( strcmp(values2[0].name, "x")==0 );
     Assert( values2[0].long_value == 14 );
     Assert( values2[0].equal == 1 );
@@ -404,13 +496,15 @@ static void test_parse_keyval_string()
                               values_required, GRIB_TYPE_DOUBLE, values3, &count);
     Assert( !err );
     Assert( count == 1 );
+    grib_print_values("print values test: values3", values3, stdout);
     Assert( strcmp(values3[0].name, "mars.level")==0 );
     free( (void*)values3[0].name );
 }
 
 static void test_dates()
 {
-    printf("Testing: dates...\n");
+    printf("Running %s ...\n", __func__);
+
     Assert( is_date_valid(1979,12, 1, 0,0,0) );
     Assert( is_date_valid(1900, 1, 1, 0,0,0) );
     Assert( is_date_valid(1964, 4, 6, 0,0,0) );
@@ -439,24 +533,228 @@ static void test_dates()
     Assert( !is_date_valid(2000, 5, 9, 0, 1, 60) );
 
     Assert( !is_date_valid(2023, 2, 29, 0,0,0) );//Feb
+}
 
+void test_scale_factor_scaled_values()
+{
+    printf("Running %s ...\n", __func__);
+
+    int err =0;
+    int64_t value, factor;
+    const int64_t scaled_value_max = 4294967295; // usually 4 octets
+    const int64_t scale_factor_max = 255; // usually 1 octet
+
+    err = compute_scaled_value_and_scale_factor(0, scaled_value_max, scale_factor_max, &value, &factor);
+    Assert(!err);
+    Assert(value == 0);
+    Assert(factor == 0);
+
+    err = compute_scaled_value_and_scale_factor(1, scaled_value_max, scale_factor_max, &value, &factor);
+    Assert(!err);
+    Assert(value == 1);
+    Assert(factor == 0);
+
+    err = compute_scaled_value_and_scale_factor(1.5, scaled_value_max, scale_factor_max, &value, &factor);
+    Assert(!err);
+    Assert(value == 15);
+    Assert(factor == 1);
+
+    err = compute_scaled_value_and_scale_factor(4.56, scaled_value_max, scale_factor_max, &value, &factor);
+    Assert(!err);
+    Assert(value == 456);
+    Assert(factor == 2);
+
+    err = compute_scaled_value_and_scale_factor(-0.003, scaled_value_max, scale_factor_max, &value, &factor);
+    Assert(!err);
+    Assert(value == -3);
+    Assert(factor == 3);
+
+    err = compute_scaled_value_and_scale_factor(145.889, scaled_value_max, scale_factor_max, &value, &factor);
+    Assert(!err);
+    Assert(value == 145889);
+    Assert(factor == 3);
+
+    err = compute_scaled_value_and_scale_factor(1111.00009, scaled_value_max, scale_factor_max, &value, &factor);
+    Assert(!err);
+    Assert(value == 111100009);
+    Assert(factor == 5);
+}
+
+void test_iarray()
+{
+    printf("Running %s ...\n", __func__);
+    grib_context* c = grib_context_get_default();
+    grib_iarray* a = grib_iarray_new(c, 10, 10);
+    grib_iarray_push(a, 42);
+    grib_iarray_push(a, 10000);
+    grib_iarray_print("iarray", a);
+
+    grib_iarray* b = grib_iarray_new(c, 1, 1);
+    grib_iarray_push(b, 0);
+    grib_iarray_push(b, -1);
+    grib_iarray_push(b, +1);
+
+    grib_viarray* va = grib_viarray_new(c, 1, 1);
+    grib_viarray_push(c, va, a);
+    grib_viarray_push(c, va, b);
+    grib_viarray_print("viarray", va);
+
+    grib_iarray_delete(a);
+    grib_iarray_delete(b);
+    grib_viarray_delete(c, va);
+}
+
+void test_darray()
+{
+    printf("Running %s ...\n", __func__);
+    grib_context* c = grib_context_get_default();
+    grib_darray* a = grib_darray_new(c, 10, 10);
+    grib_darray_push(c, a, 42.009);
+    grib_darray_push(c, a, -1.11);
+    grib_darray_push(c, a, 5099);
+    grib_darray_print("darray", a);
+
+    grib_darray* b = grib_darray_new(c, 5, 1);
+    grib_darray_push(c, b, 8);
+    grib_darray_push(c, b, 12);
+
+    grib_vdarray* va = grib_vdarray_new(c, 1, 1);
+    grib_vdarray_push(c, va, a);
+    grib_vdarray_push(c, va, b);
+    grib_vdarray_print("vdarray", va);
+
+    grib_darray_delete(c, a);
+    grib_darray_delete(c, b);
+    grib_vdarray_delete(c, va);
+}
+
+void test_sarray()
+{
+    printf("Running %s ...\n", __func__);
+    grib_context* c = grib_context_get_default();
+    grib_sarray* a = grib_sarray_new(c, 10, 10);
+
+    char ants_s[] = "ants";
+    char bugs_s[] = "bugs";
+    grib_sarray_push(c, a, ants_s);
+    grib_sarray_push(c, a, bugs_s);
+    grib_sarray_print("sarray", a);
+
+    grib_vsarray* va = grib_vsarray_new(c, 1, 1);
+    grib_vsarray_push(c, va, a);
+    grib_vsarray_print("vsarray", va);
+
+    grib_sarray_delete(c, a);
+    grib_vsarray_delete(c, va);
+}
+
+void test_codes_get_product_name()
+{
+    printf("Running %s ...\n", __func__);
+    Assert( STR_EQUAL("ANY",   codes_get_product_name(PRODUCT_ANY)) );
+    Assert( STR_EQUAL("GRIB",  codes_get_product_name(PRODUCT_GRIB)) );
+    Assert( STR_EQUAL("BUFR",  codes_get_product_name(PRODUCT_BUFR)) );
+    Assert( STR_EQUAL("GTS",   codes_get_product_name(PRODUCT_GTS)) );
+    Assert( STR_EQUAL("METAR", codes_get_product_name(PRODUCT_METAR)) );
+    Assert( STR_EQUAL("TAF",   codes_get_product_name(PRODUCT_TAF)) );
+}
+
+void test_codes_get_type_name()
+{
+    printf("Running %s ...\n", __func__);
+    Assert( STR_EQUAL("long",    codes_get_type_name(GRIB_TYPE_LONG)) );
+    Assert( STR_EQUAL("string",  grib_get_type_name(GRIB_TYPE_STRING)) );
+    Assert( STR_EQUAL("double",  grib_get_type_name(GRIB_TYPE_DOUBLE)) );
+    Assert( STR_EQUAL("bytes",   grib_get_type_name(GRIB_TYPE_BYTES)) );
+    Assert( STR_EQUAL("label",   grib_get_type_name(GRIB_TYPE_LABEL)) );
+    Assert( STR_EQUAL("section", grib_get_type_name(GRIB_TYPE_SECTION)) );
+}
+
+void test_grib2_select_PDTN()
+{
+    printf("Running %s ...\n", __func__);
+    int eps = 1;
+    int instant = 1;
+    int chemical = 1;
+    int chemical_srcsink = 1;
+    int chemical_distfn = 1;
+    int aerosol = 1;
+    int aerosol_optical = 1;
+
+    // arguments = eps instant chemical chemical_srcsink chemical_distfn aerosol aerosol_optical
+
+    // Chemicals
+    Assert( 40 == grib2_select_PDTN(!eps, instant,  chemical, 0, 0, 0, 0) );
+    Assert( 41 == grib2_select_PDTN(eps,  instant,  chemical, 0, 0, 0, 0) );
+    Assert( 42 == grib2_select_PDTN(!eps, !instant, chemical, 0, 0, 0, 0) );
+    Assert( 43 == grib2_select_PDTN(eps,  !instant, chemical, 0, 0, 0, 0) );
+
+    // Chemical source/sink
+    Assert( 76 == grib2_select_PDTN(!eps, instant,  !chemical, chemical_srcsink, 0,0,0) );
+    Assert( 77 == grib2_select_PDTN(eps,  instant,  !chemical, chemical_srcsink, 0,0,0) );
+    Assert( 78 == grib2_select_PDTN(!eps, !instant, !chemical, chemical_srcsink, 0,0,0) );
+    Assert( 79 == grib2_select_PDTN(eps,  !instant, !chemical, chemical_srcsink, 0,0,0) );
+
+    // Chemical distrib. function
+    Assert( 58 == grib2_select_PDTN(eps, instant,  0, 0, chemical_distfn, 0, 0) );
+    Assert( 68 == grib2_select_PDTN(eps, !instant,  0, 0, chemical_distfn, 0, 0) );
+    Assert( 57 == grib2_select_PDTN(!eps, instant,  0, 0, chemical_distfn, 0, 0) );
+    Assert( 67 == grib2_select_PDTN(!eps, !instant,  0, 0, chemical_distfn, 0, 0) );
+
+    // Aerosols
+    Assert( 48 == grib2_select_PDTN(!eps, instant,  0, 0, 0, aerosol, 0) );
+    Assert( 46 == grib2_select_PDTN(!eps, !instant, 0, 0, 0, aerosol, 0) );
+    Assert( 45 == grib2_select_PDTN(eps, instant,   0, 0, 0, aerosol, 0) );
+    Assert( 85 == grib2_select_PDTN(eps, !instant,  0, 0, 0, aerosol, 0) );
+
+    // Aerosol optical
+    Assert( 49 == grib2_select_PDTN(eps, instant,  0, 0, 0, 0, aerosol_optical) );
+    Assert( 48 == grib2_select_PDTN(!eps, instant, 0, 0, 0, 0, aerosol_optical) );
+
+    // Plain vanilla
+    Assert(  0 == grib2_select_PDTN(!eps, instant,  !chemical, !chemical_srcsink, !chemical_distfn, !aerosol, 0) );
+    Assert(  1 == grib2_select_PDTN(1,1,0,0,0, !aerosol,0) );
+    Assert(  8 == grib2_select_PDTN(0,0,0,0,0, !aerosol,0) );
+    Assert( 11 == grib2_select_PDTN(1,0,0,0,0, !aerosol,0) );
+
+    //printf("%d\n", grib2_select_PDTN(!eps, instant,  !chemical, !chemical_srcsink, !chemical_distfn, aerosol, 0) );
+    Assert( 1 == grib2_is_PDTN_EPS(1) );
+    Assert( 1 == grib2_is_PDTN_EPS(11) );
+    Assert( 0 == grib2_is_PDTN_EPS(0) );
+}
+
+void test_codes_get_error_message()
+{
+    printf("Running %s ...\n", __func__);
+    const char* errmsg = grib_get_error_message(6666);
+    Assert( STR_EQUAL(errmsg, "Unknown error -6666"));
 }
 
 int main(int argc, char** argv)
 {
     printf("Doing unit tests. ecCodes version = %ld\n", grib_get_api_version());
 
+    test_codes_get_error_message();
+
+    test_iarray();
+    test_darray();
+    test_sarray();
+
+    test_codes_get_product_name();
+    test_codes_get_type_name();
+
+    test_scale_factor_scaled_values();
     test_dates();
     test_logging_proc();
     test_grib_binary_search();
     test_parse_keyval_string();
 
-    test_trimming();
-    test_string_ends_with();
-
     test_get_git_sha1();
+    test_get_package_name();
     test_get_build_date();
     test_gribex_mode();
+    test_gts_header_mode();
+    test_bufr_multi_element_constant_arrays();
 
     test_concept_condition_strings();
 
@@ -483,6 +781,13 @@ int main(int argc, char** argv)
     test_grib_nearest_smaller_ieeefloat();
 
     test_string_splitting();
+    test_string_ends_with();
+    test_string_to_long();
+    test_string_trimming();
+    test_string_replace_char();
+    test_string_remove_char();
+
+    test_grib2_select_PDTN();
 
     return 0;
 }
