@@ -24,17 +24,17 @@ grib_option grib_options[] = {
     { "H", 0, "Compare only message headers. Bit-by-bit compare on. Incompatible with -c option.\n", 0, 1, 0 },
     { "R:", 0, 0, 0, 1, 0 },
     { "A:", 0, 0, 0, 1, 0 },
-    /*    {"P",0,"Compare data values using the packing error as tolerance.\n",0,1,0},*/
     { "t:", "factor", "Compare data values using factor multiplied by the tolerance specified in options -R -A.\n", 0, 1, 0 },
     { "w:", 0, 0, 0, 1, 0 },
-    { "f", 0, 0, 0, 1, 0 },
+    { "f", 0, "Forcefully compare, do not stop after first difference.\n", 0, 1, 0 },
     { "F", 0, 0, 1, 0, 0 },
     { "q", 0, 0, 1, 0, 0 },
     { "M", 0, 0, 1, 0, 0 },
     { "I", 0, 0, 1, 0, 0 },
     { "V", 0, 0, 0, 1, 0 },
     { "7", 0, 0, 0, 1, 0 },
-    { "v", 0, 0, 0, 1, 0 }
+    { "v", 0, 0, 0, 1, 0 },
+    { "h", 0, 0, 0, 1, 0 },
 };
 
 int grib_options_count = sizeof(grib_options) / sizeof(grib_option);
@@ -73,7 +73,6 @@ struct grib_error
 static grib_error* error_summary;
 static compare_double_proc compare_double;
 static double global_tolerance     = 0;
-static int packingCompare          = 0;
 static grib_string_list* blocklist = 0;
 static grib_string_list* keys_list = NULL; /* Used to determine rank of key */
 static int isLeafKey               = 0;    /* 0 if key is top-level, 1 if key has no children attributes */
@@ -85,7 +84,6 @@ static int compare_attribute(grib_handle* handle1, grib_handle* handle2, grib_ru
 
 static int error     = 0;
 static int count     = 0;
-static int lastPrint = 0;
 static int force     = 0;
 
 /* ECC-651: Boolean 'two_way' set to 1 when '-2' option used */
@@ -237,7 +235,7 @@ int grib_tool_before_getopt(grib_runtime_options* options)
 
 int grib_tool_init(grib_runtime_options* options)
 {
-    int ret               = 0, i;
+    int ret = 0;
     grib_context* context = grib_context_get_default();
 
     options->strict = 1;
@@ -316,16 +314,6 @@ int grib_tool_init(grib_runtime_options* options)
     compare_double   = &compare_double_absolute;
     if (grib_options_on("R:")) {
         global_tolerance = 0;
-        for (i = 0; i < options->tolerance_count; i++) {
-            if (!strcmp((options->tolerance[i]).name, "all")) {
-                global_tolerance = (options->tolerance[i]).double_value;
-                break;
-            }
-            if (!strcmp((options->tolerance[i]).name, "global")) {
-                global_tolerance = (options->tolerance[i]).double_value;
-                break;
-            }
-        }
         compare_double  = &compare_double_relative;
         compareAbsolute = 0;
     }
@@ -338,18 +326,14 @@ int grib_tool_init(grib_runtime_options* options)
             global_tolerance = atof(grib_options_get_option("A:"));
         }
     }
-    if (grib_options_on("P")) {
-        packingCompare = 1;
-        compare_double = &compare_double_absolute;
-    }
 
     if (grib_options_on("t:"))
         tolerance_factor = atof(grib_options_get_option("t:"));
 
     if (grib_options_on("R:")) {
-        char* sarg               = grib_options_get_option("R:");
+        char* sarg = grib_options_get_option("R:");
         options->tolerance_count = MAX_KEYS;
-        ret                      = parse_keyval_string(tool_name, sarg, 1, GRIB_TYPE_DOUBLE, options->tolerance, &(options->tolerance_count));
+        ret = parse_keyval_string(tool_name, sarg, 1, GRIB_TYPE_DOUBLE, options->tolerance, &(options->tolerance_count));
         if (ret == GRIB_INVALID_ARGUMENT) {
             usage();
             exit(1);
@@ -394,7 +378,6 @@ int grib_tool_new_file_action(grib_runtime_options* options, grib_tools_file* fi
 static void printInfo(grib_handle* h)
 {
     printf("== %d == DIFFERENCE == ", count);
-    lastPrint = count;
 }
 
 static void print_index_key_values(grib_index* index, int icounter, const char* error_message)
@@ -530,11 +513,6 @@ int grib_tool_skip_handle(grib_runtime_options* options, grib_handle* h)
     return 0;
 }
 
-void grib_tool_print_key_values(grib_runtime_options* options, grib_handle* h)
-{
-    grib_print_key_values(options, h);
-}
-
 int grib_tool_finalise_action(grib_runtime_options* options)
 {
     grib_error* e   = error_summary;
@@ -703,11 +681,9 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
 
     char *sval1 = NULL, *sval2 = NULL;
     char **svals1 = NULL, **svals2 = NULL;
-    unsigned char *uval1 = NULL, *uval2 = NULL;
     double *dval1 = NULL, *dval2 = NULL;
     long *lval1 = NULL, *lval2 = NULL;
     double maxdiff       = 0;
-    double packingError1 = 0, packingError2 = 0;
     double value_tolerance = 0;
     grib_context* c        = handle1->context;
     const char* first_str  = (handles_swapped == 0 ? "1st" : "2nd");
@@ -720,7 +696,7 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
 
     if (type1 == GRIB_TYPE_UNDEFINED && (err = grib_get_native_type(handle1, name, &type1)) != GRIB_SUCCESS) {
         printInfo(handle1);
-        printf("Oops... cannot get type of [%s] in %s field: %s\n", name, first_str, grib_get_error_message(err));
+        printf("Error: cannot get type of [%s] in %s field: %s\n", name, first_str, grib_get_error_message(err));
         save_error(c, name);
         return err;
     }
@@ -733,7 +709,7 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
             return err;
         }
         printInfo(handle1);
-        printf("Oops... cannot get type of [%s] in %s field: %s\n", name, second_str, grib_get_error_message(err));
+        printf("Error: cannot get type of [%s] in %s field: %s\n", name, second_str, grib_get_error_message(err));
         save_error(c, name);
         return err;
     }
@@ -753,7 +729,7 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
 
     if ((err = grib_get_size(handle1, name, &len1)) != GRIB_SUCCESS) {
         printInfo(handle1);
-        printf("Oops... cannot get size of [%s] in %s field: %s\n", name, first_str, grib_get_error_message(err));
+        printf("Error: cannot get size of [%s] in %s field: %s\n", name, first_str, grib_get_error_message(err));
         save_error(c, name);
         return err;
     }
@@ -767,7 +743,7 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
         }
 
         printInfo(handle1);
-        printf("Oops... cannot get size of [%s] in %s field: %s\n", name, second_str, grib_get_error_message(err));
+        printf("Error: cannot get size of [%s] in %s field: %s\n", name, second_str, grib_get_error_message(err));
         save_error(c, name);
         return err;
     }
@@ -783,21 +759,18 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
           return GRIB_COUNT_MISMATCH;
         } */
 
-    if (options->mode != MODE_BUFR) {
+    Assert(options->mode == MODE_BUFR);
         /* TODO: Ignore missing values for keys in BUFR. Not yet implemented */
-        isMissing1 = ((grib_is_missing(handle1, name, &err1) == 1) && (err1 == 0)) ? 1 : 0;
-        isMissing2 = ((grib_is_missing(handle2, name, &err2) == 1) && (err2 == 0)) ? 1 : 0;
-    }
+        //isMissing1 = ((grib_is_missing(handle1, name, &err1) == 1) && (err1 == 0)) ? 1 : 0;
+        //isMissing2 = ((grib_is_missing(handle2, name, &err2) == 1) && (err2 == 0)) ? 1 : 0;
 
     if ((isMissing1 == 1) && (isMissing2 == 1)) {
-        if (verbose)
-            printf(" is set to missing in both fields\n");
+        // if (verbose) printf(" is set to missing in both fields\n");
         return GRIB_SUCCESS;
     }
 
     if (isMissing1 == 1) {
-        if (verbose)
-            printf(" is set to missing in %s field\n", first_str);
+        // if (verbose) printf(" is set to missing in %s field\n", first_str);
         printInfo(handle1);
         printf("%s is set to missing in %s field is not missing in %s field\n", name, first_str, second_str);
         err1 = GRIB_VALUE_MISMATCH;
@@ -806,8 +779,7 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
     }
 
     if (isMissing2 == 1) {
-        if (verbose)
-            printf(" is set to missing in %s field\n", first_str);
+        // if (verbose) printf(" is set to missing in %s field\n", first_str);
         printInfo(handle1);
         printf("%s is set to missing in %s field is not missing in %s field\n", name, second_str, first_str);
         err1 = GRIB_VALUE_MISMATCH;
@@ -832,13 +804,13 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
                 sval2         = (char*)grib_context_malloc(handle2->context, slen2 * sizeof(char));
                 if ((err1 = grib_get_string(handle1, name, sval1, &slen1)) != GRIB_SUCCESS) {
                     printInfo(handle1);
-                    printf("Oops... cannot get string value of [%s] in %s field: %s\n",
+                    printf("Error: cannot get string value of [%s] in %s field: %s\n",
                            name, first_str, grib_get_error_message(err1));
                     save_error(c, name);
                 }
                 if ((err2 = grib_get_string(handle2, name, sval2, &slen2)) != GRIB_SUCCESS) {
                     printInfo(handle1);
-                    printf("Oops... cannot get string value of [%s] in %s field: %s\n",
+                    printf("Error: cannot get string value of [%s] in %s field: %s\n",
                            name, second_str, grib_get_error_message(err2));
                     save_error(c, name);
                 }
@@ -861,13 +833,13 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
 
                 if ((err1 = grib_get_string_array(handle1, name, svals1, &len1)) != GRIB_SUCCESS) {
                     printInfo(handle1);
-                    printf("Oops... cannot get string value of [%s] in %s field: %s\n",
+                    printf("Error: cannot get string value of [%s] in %s field: %s\n",
                            name, first_str, grib_get_error_message(err1));
                     save_error(c, name);
                 }
                 if ((err2 = grib_get_string_array(handle2, name, svals2, &len2)) != GRIB_SUCCESS) {
                     printInfo(handle1);
-                    printf("Oops... cannot get string value of [%s] in %s field: %s\n",
+                    printf("Error: cannot get string value of [%s] in %s field: %s\n",
                            name, second_str, grib_get_error_message(err2));
                     save_error(c, name);
                 }
@@ -916,14 +888,14 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
 
             if ((err1 = grib_get_long_array(handle1, name, lval1, &len1)) != GRIB_SUCCESS) {
                 printInfo(handle1);
-                printf("Oops... cannot get long value of [%s] in %s field: %s\n",
+                printf("Error: cannot get long value of [%s] in %s field: %s\n",
                        name, first_str, grib_get_error_message(err1));
                 save_error(c, name);
             }
 
             if ((err2 = grib_get_long_array(handle2, name, lval2, &len2)) != GRIB_SUCCESS) {
                 printInfo(handle1);
-                printf("Oops... cannot get long value of [%s] in %s field: %s\n",
+                printf("Error: cannot get long value of [%s] in %s field: %s\n",
                        name, second_str, grib_get_error_message(err2));
                 save_error(c, name);
             }
@@ -1001,14 +973,14 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
 
             if ((err1 = grib_get_double_array(handle1, name, dval1, &len1)) != GRIB_SUCCESS) {
                 printInfo(handle1);
-                printf("Oops... cannot get double value of [%s] in %s field: %s\n",
+                printf("Error: cannot get double value of [%s] in %s field: %s\n",
                        name, first_str, grib_get_error_message(err1));
                 save_error(c, name);
             }
 
             if ((err2 = grib_get_double_array(handle2, name, dval2, &len2)) != GRIB_SUCCESS) {
                 printInfo(handle1);
-                printf("Oops... cannot get double value of [%s] in %s field: %s\n",
+                printf("Error: cannot get double value of [%s] in %s field: %s\n",
                        name, second_str, grib_get_error_message(err2));
                 save_error(c, name);
             }
@@ -1019,28 +991,13 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
                 save_error(c, name);
             }
             if (err1 == GRIB_SUCCESS && err2 == GRIB_SUCCESS && len1 == len2) {
-                int imaxdiff;
+                int imaxdiff = 0;
                 double diff;
                 double *pv1, *pv2;
                 maxdiff   = 0;
-                imaxdiff  = 0;
                 countdiff = 0;
                 pv1       = dval1;
                 pv2       = dval2;
-                /* if (isangle) {
-                    dnew1 = *dval1;
-                    dnew2 = *dval2;
-                    pv1   = &dnew1;
-                    pv2   = &dnew2;
-                    if (*dval1 < 0)
-                        dnew1 += 360.0;
-                    if (*dval2 < 0)
-                        dnew2 += 360.0;
-                    if (*dval1 > 360)
-                        dnew1 -= 360.0;
-                    if (*dval2 > 360)
-                        dnew2 -= 360.0;
-                } */
                 value_tolerance *= tolerance_factor;
                 if (verbose)
                     printf("  (%d values) tolerance=%g\n", (int)len1, value_tolerance);
@@ -1079,8 +1036,6 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
                             grib_context_free(c, svalA);
                             grib_context_free(c, svalB);
                         }
-                        if (packingError2 != 0 || packingError1 != 0)
-                            printf(" packingError: [%g] [%g]", packingError1, packingError2);
 
                         printf("\n");
                     }
@@ -1113,69 +1068,14 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
             break;
 
         case GRIB_TYPE_BYTES:
-            if (verbose)
-                printf(" as bytes\n");
-            if (options->mode == MODE_BUFR)
-                return 0;
-            if (len1 < 2)
-                len1 = 512;
-            if (len2 < 2)
-                len2 = 512;
-            uval1 = (unsigned char*)grib_context_malloc(handle1->context, len1 * sizeof(unsigned char));
-            uval2 = (unsigned char*)grib_context_malloc(handle2->context, len2 * sizeof(unsigned char));
-
-            if ((err1 = grib_get_bytes(handle1, name, uval1, &len1)) != GRIB_SUCCESS) {
-                printInfo(handle1);
-                save_error(c, name);
-                printf("Oops... cannot get bytes value of [%s] in %s field: %s\n",
-                       name, first_str, grib_get_error_message(err1));
-            }
-
-            if ((err2 = grib_get_bytes(handle2, name, uval2, &len2)) != GRIB_SUCCESS) {
-                printInfo(handle1);
-                save_error(c, name);
-                printf("Oops... cannot get bytes value of [%s] in %s field: %s\n",
-                       name, second_str, grib_get_error_message(err2));
-            }
-
-            if (err1 == GRIB_SUCCESS && err2 == GRIB_SUCCESS) {
-                if (memcmp(uval1, uval2, len1) != 0) {
-                    for (i = 0; i < len1; i++) {
-                        if (uval1[i] != uval2[i]) {
-                            printInfo(handle1);
-                            save_error(c, name);
-                            if (len1 == 1)
-                                printf("[%s] byte values are different: [%02x] and [%02x]\n",
-                                       name, uval1[i], uval2[i]);
-                            else
-                                printf("[%s] byte value %d of %ld are different: [%02x] and [%02x]\n",
-                                       name, i, (long)len1, uval1[i], uval2[i]);
-
-                            err1 = GRIB_VALUE_MISMATCH;
-                            break;
-                        }
-                    }
-                    err1 = GRIB_VALUE_MISMATCH;
-                }
-            }
-
-            grib_context_free(handle1->context, uval1);
-            grib_context_free(handle2->context, uval2);
-
-            if (err1)
-                return err1;
-            if (err2)
-                return err2;
+            return 0; // Not in BUFR
             break;
 
         case GRIB_TYPE_LABEL:
-            if (verbose)
-                printf(" as label\n");
             break;
 
         default:
-            if (verbose)
-                printf("\n");
+            if (verbose) printf("\n");
             printInfo(handle1);
             save_error(c, name);
             printf("Cannot compare [%s], unsupported type %d\n", name, type1);
@@ -1334,12 +1234,11 @@ static int compare_handles(grib_handle* handle1, grib_handle* handle2, grib_runt
             if (options->compare[i].type == CODES_NAMESPACE) {
                 iter = grib_keys_iterator_new(handle1, 0, options->compare[i].name);
                 if (!iter) {
-                    grib_context_log(handle1->context, GRIB_LOG_ERROR, "unable to get iterator");
+                    grib_context_log(handle1->context, GRIB_LOG_ERROR, "unable to create the BUFR keys iterator");
                     exit(1);
                 }
                 while (grib_keys_iterator_next(iter)) {
                     name = grib_keys_iterator_get_name(iter);
-                    /*printf("----- comparing %s\n",name);*/
 
                     if (blocklisted(name))
                         continue;
@@ -1358,11 +1257,9 @@ static int compare_handles(grib_handle* handle1, grib_handle* handle2, grib_runt
     else {
         const void *msg1 = NULL, *msg2 = NULL;
         size_t size1 = 0, size2 = 0;
-        int memcmp_ret = 0;
-        /* int ii=0; */
         GRIB_CHECK_NOLINE(grib_get_message(handle1, &msg1, &size1), 0);
         GRIB_CHECK_NOLINE(grib_get_message(handle2, &msg2, &size2), 0);
-        if (size1 == size2 && !(memcmp_ret = memcmp(msg1, msg2, size1))) {
+        if (size1 == size2 && 0 == memcmp(msg1, msg2, size1)) {
             return 0;
         }
 #if defined(BUFR_COMPARE_BYTES)
@@ -1395,12 +1292,11 @@ static int compare_handles(grib_handle* handle1, grib_handle* handle2, grib_runt
                     iter = grib_keys_iterator_new(handle1, 0, options->compare[i].name);
                     if (!iter) {
                         grib_context_log(handle1->context, GRIB_LOG_ERROR,
-                                         "ERROR: unable to get keys iterator for %s", options->compare[i].name);
+                                         "unable to create the BUFR keys iterator for %s", options->compare[i].name);
                         exit(1);
                     }
                     while (grib_keys_iterator_next(iter)) {
                         name = grib_keys_iterator_get_name(iter);
-                        /*printf("----- comparing %s\n",name);*/
 
                         if (blocklisted(name))
                             continue;
