@@ -20,11 +20,10 @@ grib_option grib_options[] = {
     { "S:", "start", "First field to be processed.\n", 0, 1, 0 },
     { "E:", "end", "Last field to be processed.\n", 0, 1, 0 },
     { "a", 0, "-c option modifier. The keys listed with the option -c will be added to the list of keys compared without -c.\n", 0, 1, 0 },
-    /*{"H",0,"Compare only message headers. Bit-by-bit compare on. Incompatible with -c option.\n",0,1,0},*/
     /*{"R:",0,0,0,1,0},*/
     /*{"A:",0,0,0,1,0},*/
     { "w:", 0, 0, 0, 1, 0 },
-    { "f", 0, 0, 0, 1, 0 },
+    { "f", 0, "Forcefully compare, do not stop after first difference.\n", 0, 1, 0 },
     { "F", 0, 0, 1, 0, 0 },
     { "q", 0, 0, 1, 0, 0 },
     { "I", 0, 0, 1, 0, 0 },
@@ -42,9 +41,7 @@ const char* tool_description =
 
 const char* tool_name = "gts_compare";
 const char* tool_online_doc = NULL;
-const char* tool_usage =
-    "[options] "
-    "file file";
+const char* tool_usage = "[options] file file";
 
 
 GRIB_INLINE static int grib_inline_strcmp(const char* a, const char* b)
@@ -77,11 +74,9 @@ static int compare_handles(grib_handle* h1, grib_handle* h2, grib_runtime_option
 
 int error               = 0;
 int count               = 0;
-int lastPrint           = 0;
 int force               = 0;
 double maxAbsoluteError = 1e-19;
 int onlyListed          = 1;
-int headerMode          = 0;
 int morein1             = 0;
 int morein2             = 0;
 int listFromCommandLine = 0;
@@ -157,9 +152,6 @@ int grib_tool_before_getopt(grib_runtime_options* options)
 
 int grib_tool_init(grib_runtime_options* options)
 {
-    int ret               = 0;
-    int nfiles            = 1;
-    char orderby[]        = "md5Headers";
     grib_context* context = grib_context_get_default();
 
     options->strict = 1;
@@ -190,17 +182,8 @@ int grib_tool_init(grib_runtime_options* options)
     else
         onlyListed = 1;
 
-    if (grib_options_on("H"))
-        headerMode = 1;
-    else
-        headerMode = 0;
-
-    if (grib_options_on("H") && grib_options_on("c:")) {
-        printf("Error: -H and -c options are incompatible. Choose one of the two please.\n");
-        exit(1);
-    }
     if (grib_options_on("a") && !grib_options_on("c:")) {
-        printf("Error: -a option requires -c option. Please define a list of keys with the -c option.\n");
+        fprintf(stderr, "Error: -a option requires -c option. Please define a list of keys with the -c option.\n");
         exit(1);
     }
 
@@ -218,27 +201,12 @@ int grib_tool_init(grib_runtime_options* options)
         context->blocklist = blocklist;
     }
 
-    if (grib_options_on("r")) {
-        const char* filename[1];
-        filename[0]      = options->infile_extra->name;
-        options->random  = 1;
-        options->orderby = strdup(orderby);
-        options->idx     = grib_fieldset_new_from_files(context, filename,
-                                                    nfiles, 0, 0, 0, orderby, &ret);
-        if (ret) {
-            printf("unable to create index for input file %s (%s)",
-                   options->infile_extra->name, grib_get_error_message(ret));
-            exit(ret);
-        }
-    }
-    else {
-        options->random             = 0;
-        options->infile_extra->file = fopen(options->infile_extra->name, "r");
+    options->random             = 0;
+    options->infile_extra->file = fopen(options->infile_extra->name, "r");
 
-        if (!options->infile_extra->file) {
-            perror(options->infile_extra->name);
-            exit(1);
-        }
+    if (!options->infile_extra->file) {
+        perror(options->infile_extra->name);
+        exit(1);
     }
 
     if (grib_options_on("t:"))
@@ -276,20 +244,6 @@ int grib_tool_new_file_action(grib_runtime_options* options, grib_tools_file* fi
 static void printInfo(grib_handle* h)
 {
     printf("== %d == DIFFERENCE == ", count);
-    lastPrint = count;
-}
-
-static void print_index_key_values(grib_index* index, int cnt, const char* error_message)
-{
-    grib_index_key* keys = index->keys;
-    printf("== %d == ", cnt);
-    if (error_message)
-        printf("%s == ", error_message);
-    while (keys) {
-        printf("%s=%s ", keys->name, keys->value);
-        keys = keys->next;
-    }
-    printf("\n");
 }
 
 static grib_handle* gts_handle_new_from_file_x(
@@ -303,54 +257,7 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h)
     int err = 0;
     count++;
 
-    if (options->through_index) {
-        grib_index* idx1 = options->index1;
-        verbose          = 0;
-        counter++;
-
-        if (start > 0 && counter < start)
-            return 0;
-        if (end > 0 && counter > end) {
-            options->stop = 1;
-            return 0;
-        }
-
-        grib_index_search_same(idx1, h);
-        global_handle = codes_new_from_index(idx1, CODES_GTS, &err);
-        if (options->verbose) {
-            off_t offset   = 0;
-            char* filename = grib_get_field_file(options->index2, &offset);
-            printf("file1=\"%s\" ", filename);
-            filename = grib_get_field_file(options->index1, &offset);
-            printf("file2=\"%s\" \n", filename);
-            print_index_key_values(options->index1, counter, NULL);
-        }
-
-        if (!global_handle) {
-            if (!options->verbose)
-                print_index_key_values(idx1, counter, "NOT FOUND ");
-        }
-
-        if (!global_handle || err != GRIB_SUCCESS) {
-            morein1++;
-            grib_handle_delete(global_handle);
-            return 0;
-        }
-
-        if (compare_handles(h, global_handle, options)) {
-            error++;
-            if (!force)
-                exit(1);
-        }
-
-        grib_handle_delete(global_handle);
-
-        return 0;
-    }
-    else if (options->random)
-        global_handle = grib_fieldset_next_handle(options->idx, &err);
-    else
-        global_handle = gts_handle_new_from_file_x(h->context, options->infile_extra->file, options->mode, 0, &err);
+    global_handle = gts_handle_new_from_file_x(h->context, options->infile_extra->file, options->mode, 0, &err);
 
     if (!global_handle || err != GRIB_SUCCESS) {
         morein2++;
@@ -373,7 +280,7 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h)
 int grib_tool_skip_handle(grib_runtime_options* options, grib_handle* h)
 {
     int err = 0;
-    if (!options->through_index && !options->random) {
+    if (!options->through_index) {
         global_handle = gts_new_from_file(h->context, options->infile_extra->file, &err);
 
         if (!global_handle || err != GRIB_SUCCESS)
@@ -388,11 +295,6 @@ int grib_tool_skip_handle(grib_runtime_options* options, grib_handle* h)
     return 0;
 }
 
-void grib_tool_print_key_values(grib_runtime_options* options, grib_handle* h)
-{
-    grib_print_key_values(options, h);
-}
-
 int grib_tool_finalise_action(grib_runtime_options* options)
 {
     grib_error* e   = error_summary;
@@ -430,10 +332,6 @@ int grib_tool_finalise_action(grib_runtime_options* options)
 
         printf("##\n## %d different messages out of %d\n\n", error, count);
     }
-    if (options->through_index) {
-        grib_index_delete(options->index1);
-        grib_index_delete(options->index2);
-    }
 
     if (error != 0)
         exit(1);
@@ -473,19 +371,15 @@ static void save_error(grib_context* c, const char* key)
     }
 }
 
-static int compare_values(grib_runtime_options* options, grib_handle* h1, grib_handle* h2, const char* name, int type)
+static int compare_values(const grib_runtime_options* options, grib_handle* h1, grib_handle* h2, const char* name, int type)
 {
-    size_t len1 = 0;
-    size_t len2 = 0;
-    int err     = 0;
-    int err1;
-    int err2;
+    size_t len1 = 0, len2 = 0;
+    int err = 0, err1 = 0, err2 = 0;
     int type1, type2;
     int countdiff;
-    int isMissing1 = 0, isMissing2 = 0;
+    // int isMissing1 = 0, isMissing2 = 0;
 
     char *sval1 = NULL, *sval2 = NULL;
-    unsigned char *uval1 = NULL, *uval2 = NULL;
     long *lval1 = NULL, *lval2 = NULL;
     grib_context* c = h1->context;
 
@@ -496,7 +390,7 @@ static int compare_values(grib_runtime_options* options, grib_handle* h1, grib_h
 
     if (type1 == GRIB_TYPE_UNDEFINED && (err = grib_get_native_type(h1, name, &type1)) != GRIB_SUCCESS) {
         printInfo(h1);
-        printf("Oops... cannot get type of [%s] in 1st field: %s\n", name, grib_get_error_message(err));
+        printf("Error: cannot get type of [%s] in 1st field: %s\n", name, grib_get_error_message(err));
         save_error(c, name);
         return err;
     }
@@ -509,31 +403,20 @@ static int compare_values(grib_runtime_options* options, grib_handle* h1, grib_h
             return err;
         }
         printInfo(h1);
-        printf("Oops... cannot get type of [%s] in 2nd field: %s\n", name, grib_get_error_message(err));
+        printf("Error: cannot get type of [%s] in 2nd field: %s\n", name, grib_get_error_message(err));
         save_error(c, name);
         return err;
     }
 
-    /*
-  if(type1 != type2)
-  {
-    printInfo(h1);
-    printf("Warning, [%s] has different types: 1st field: [%s], 2nd field: [%s]\n",
-        name,grib_get_type_name(type1),grib_get_type_name(type2));
-    return GRIB_TYPE_MISMATCH;
-  }
-     */
-
     if (type1 == GRIB_TYPE_LABEL)
         return err;
 
     if (type1 == GRIB_TYPE_SECTION)
         return err;
 
-
     if ((err = grib_get_size(h1, name, &len1)) != GRIB_SUCCESS) {
         printInfo(h1);
-        printf("Oops... cannot get size of [%s] in 1st field: %s\n", name, grib_get_error_message(err));
+        printf("Error: cannot get size of [%s] in 1st field: %s\n", name, grib_get_error_message(err));
         save_error(c, name);
         return err;
     }
@@ -547,57 +430,18 @@ static int compare_values(grib_runtime_options* options, grib_handle* h1, grib_h
         }
 
         printInfo(h1);
-        printf("Oops... cannot get size of [%s] in 2nd field: %s\n", name, grib_get_error_message(err));
+        printf("Error: cannot get size of [%s] in 2nd field: %s\n", name, grib_get_error_message(err));
         save_error(c, name);
         return err;
     }
-
-    /*
-  if(len1 != len2 && type1 != GRIB_TYPE_STRING)
-  {
-    printInfo(h1);
-    printf("[%s] has different size: 1st field: %ld, 2nd field: %ld\n",name,(long)len1,(long)len2);
-    save_error(c,name);
-    return GRIB_COUNT_MISMATCH;
-  }
-     */
-
-    if (options->mode != MODE_GTS) {
+    Assert(options->mode == MODE_GTS);
         /* TODO: Ignore missing values for keys in GTS. Not yet implemented */
-        isMissing1 = ((grib_is_missing(h1, name, &err1) == 1) && (err1 == 0)) ? 1 : 0;
-        isMissing2 = ((grib_is_missing(h2, name, &err2) == 1) && (err2 == 0)) ? 1 : 0;
-    }
-
-    if ((isMissing1 == 1) && (isMissing2 == 1)) {
-        if (verbose)
-            printf(" is set to missing in both fields\n");
-        return GRIB_SUCCESS;
-    }
-
-    if (isMissing1 == 1) {
-        if (verbose)
-            printf(" is set to missing in 1st field\n");
-        printInfo(h1);
-        printf("%s is set to missing in 1st field but is not missing in 2nd field\n", name);
-        err1 = GRIB_VALUE_MISMATCH;
-        save_error(c, name);
-        return GRIB_VALUE_MISMATCH;
-    }
-
-    if (isMissing2 == 1) {
-        if (verbose)
-            printf(" is set to missing in 1st field\n");
-        printInfo(h1);
-        printf("%s is set to missing in 2nd field but is not missing in 1st field\n", name);
-        err1 = GRIB_VALUE_MISMATCH;
-        save_error(c, name);
-        return GRIB_VALUE_MISMATCH;
-    }
+        //isMissing1 = ((grib_is_missing(h1, name, &err1) == 1) && (err1 == 0)) ? 1 : 0;
+        //isMissing2 = ((grib_is_missing(h2, name, &err2) == 1) && (err2 == 0)) ? 1 : 0;
 
     switch (type1) {
         case GRIB_TYPE_STRING:
-            if (verbose)
-                printf(" as string\n");
+            if (verbose) printf(" as string\n");
             grib_get_string_length(h1, name, &len1);
             grib_get_string_length(h2, name, &len2);
             sval1 = (char*)grib_context_malloc(h1->context, len1 * sizeof(char));
@@ -605,14 +449,14 @@ static int compare_values(grib_runtime_options* options, grib_handle* h1, grib_h
 
             if ((err1 = grib_get_string(h1, name, sval1, &len1)) != GRIB_SUCCESS) {
                 printInfo(h1);
-                printf("Oops... cannot get string value of [%s] in 1st field: %s\n",
+                printf("Error: cannot get string value of [%s] in 1st field: %s\n",
                        name, grib_get_error_message(err1));
                 save_error(c, name);
             }
 
             if ((err2 = grib_get_string(h2, name, sval2, &len2)) != GRIB_SUCCESS) {
                 printInfo(h1);
-                printf("Oops... cannot get string value of [%s] in 2nd field: %s\n",
+                printf("Error: cannot get string value of [%s] in 2nd field: %s\n",
                        name, grib_get_error_message(err2));
                 save_error(c, name);
             }
@@ -638,22 +482,21 @@ static int compare_values(grib_runtime_options* options, grib_handle* h1, grib_h
             break;
 
         case GRIB_TYPE_LONG:
-            if (verbose)
-                printf(" as long\n");
+            if (verbose) printf(" as long\n");
 
             lval1 = (long*)grib_context_malloc(h1->context, len1 * sizeof(long));
             lval2 = (long*)grib_context_malloc(h2->context, len2 * sizeof(long));
 
             if ((err1 = grib_get_long_array(h1, name, lval1, &len1)) != GRIB_SUCCESS) {
                 printInfo(h1);
-                printf("Oops... cannot get long value of [%s] in 1st field: %s\n",
+                printf("Error: cannot get long value of [%s] in 1st field: %s\n",
                        name, grib_get_error_message(err1));
                 save_error(c, name);
             }
 
             if ((err2 = grib_get_long_array(h2, name, lval2, &len2)) != GRIB_SUCCESS) {
                 printInfo(h1);
-                printf("Oops... cannot get long value of [%s] in 2nd field: %s\n",
+                printf("Error: cannot get long value of [%s] in 2nd field: %s\n",
                        name, grib_get_error_message(err2));
                 save_error(c, name);
             }
@@ -699,80 +542,24 @@ static int compare_values(grib_runtime_options* options, grib_handle* h1, grib_h
             break;
 
         case GRIB_TYPE_BYTES:
-            if (verbose)
-                printf(" as bytes\n");
-            if (options->mode == MODE_GTS)
-                return 0;
-            if (len1 < 2)
-                len1 = 512;
-            if (len2 < 2)
-                len2 = 512;
-            uval1 = (unsigned char*)grib_context_malloc(h1->context, len1 * sizeof(unsigned char));
-            uval2 = (unsigned char*)grib_context_malloc(h2->context, len2 * sizeof(unsigned char));
-
-            if ((err1 = grib_get_bytes(h1, name, uval1, &len1)) != GRIB_SUCCESS) {
-                printInfo(h1);
-                save_error(c, name);
-                printf("Oops... cannot get bytes value of [%s] in 1st field: %s\n",
-                       name, grib_get_error_message(err1));
-            }
-
-            if ((err2 = grib_get_bytes(h2, name, uval2, &len2)) != GRIB_SUCCESS) {
-                printInfo(h1);
-                save_error(c, name);
-                printf("Oops... cannot get bytes value of [%s] in 2nd field: %s\n",
-                       name, grib_get_error_message(err2));
-            }
-
-            if (err1 == GRIB_SUCCESS && err2 == GRIB_SUCCESS) {
-                if (memcmp(uval1, uval2, len1) != 0) {
-                    size_t i;
-                    for (i = 0; i < len1; i++)
-                        if (uval1[i] != uval2[i]) {
-                            printInfo(h1);
-                            save_error(c, name);
-                            if (len1 == 1)
-                                printf("[%s] byte values are different: [%02x] and [%02x]\n",
-                                       name, uval1[i], uval2[i]);
-                            else
-                                printf("[%s] byte value %zu of %ld are different: [%02x] and [%02x]\n",
-                                       name, i, (long)len1, uval1[i], uval2[i]);
-
-                            err1 = GRIB_VALUE_MISMATCH;
-                            break;
-                        }
-                    err1 = GRIB_VALUE_MISMATCH;
-                }
-            }
-
-            grib_context_free(h1->context, uval1);
-            grib_context_free(h2->context, uval2);
-
-            if (err1)
-                return err1;
-            if (err2)
-                return err2;
-            break;
+            // We do not want to compare the message itself
+            return 0;
 
         case GRIB_TYPE_LABEL:
-            if (verbose)
-                printf(" as label\n");
             break;
 
         default:
-            if (verbose)
-                printf("\n");
+            if (verbose) printf("\n");
             printInfo(h1);
             save_error(c, name);
-            printf("Cannot compare [%s], unsupported type %d\n", name, type1);
+            fprintf(stderr, "Cannot compare [%s], unsupported type %d\n", name, type1);
             return GRIB_UNABLE_TO_COMPARE_ACCESSORS;
-            break;
     }
 
     return GRIB_SUCCESS;
 }
 
-static int compare_all_dump_keys(grib_handle* h1, grib_handle* h2, grib_runtime_options* options, int* err)
+static int compare_all_dump_keys(grib_handle* h1, grib_handle* h2, grib_runtime_options* options, int* pErr)
 {
     int ret                  = 0;
     const char* name         = NULL;
@@ -780,21 +567,20 @@ static int compare_all_dump_keys(grib_handle* h1, grib_handle* h2, grib_runtime_
     iter                     = grib_keys_iterator_new(h1, 0, NULL);
 
     if (!iter) {
-        printf("ERROR: unable to get iterator\n");
+        fprintf(stderr, "Error: unable to get keys iterator\n");
         exit(1);
     }
 
     while (grib_keys_iterator_next(iter)) {
         grib_accessor* xa = grib_keys_iterator_get_accessor(iter);
         name              = grib_keys_iterator_get_name(iter);
-        /* printf("----- comparing %s\n",name); */
 
         if (blocklisted(name))
             continue;
         if (xa == NULL || (xa->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0)
             continue;
         if (compare_values(options, h1, h2, name, GRIB_TYPE_UNDEFINED)) {
-            err++;
+            (*pErr)++;
             write_messages(h1, h2);
             ret = 1;
         }
@@ -811,68 +597,18 @@ static int compare_handles(grib_handle* h1, grib_handle* h2, grib_runtime_option
     grib_keys_iterator* iter = NULL;
     const char* name         = NULL;
 
-    /* mask only if no -c option or headerMode (-H)*/
-    if (blocklist && (!listFromCommandLine || headerMode)) {
-        grib_string_list* nextb = blocklist;
-        while (nextb) {
-            grib_clear(h1, nextb->value);
-            grib_clear(h2, nextb->value);
-            nextb = nextb->next;
-        }
-    }
-
-    if (headerMode) {
-        const void *msg1 = NULL, *msg2 = NULL;
-        size_t size1 = 0, size2 = 0;
-        grib_handle *h11, *h22;
-        GRIB_CHECK_NOLINE(grib_get_message_headers(h1, &msg1, &size1), 0);
-        GRIB_CHECK_NOLINE(grib_get_message_headers(h2, &msg2, &size2), 0);
-        if (size1 == size2 && !memcmp(msg1, msg2, size1))
-            return 0;
-
-        err = 0;
-        h11 = grib_handle_new_from_partial_message(h1->context, msg1, size1);
-        h22 = grib_handle_new_from_partial_message(h1->context, msg2, size2);
-
-        iter = grib_keys_iterator_new(h11,
-                                      GRIB_KEYS_ITERATOR_SKIP_COMPUTED, NULL);
-
-        if (!iter) {
-            printf("ERROR: unable to get iterator\n");
-            exit(1);
-        }
-
-        while (grib_keys_iterator_next(iter)) {
-            name = grib_keys_iterator_get_name(iter);
-            /*printf("----- comparing %s\n",name);*/
-
-            if (blocklisted(name))
-                continue;
-            if (compare_values(options, h11, h22, name, GRIB_TYPE_UNDEFINED)) {
-                err++;
-                write_messages(h11, h22);
-            }
-        }
-
-        grib_keys_iterator_delete(iter);
-        grib_handle_delete(h11);
-        grib_handle_delete(h22);
-        return err;
-    }
-
     if (listFromCommandLine && onlyListed) {
         for (i = 0; i < options->compare_count; i++) {
             if (blocklisted((char*)options->compare[i].name))
                 continue;
-            if (options->compare[i].type == GRIB_NAMESPACE) {
+            if (options->compare[i].type == CODES_NAMESPACE) {
                 iter = grib_keys_iterator_new(h1, 0, options->compare[i].name);
                 if (!iter) {
-                    printf("ERROR: unable to get iterator\n");
+                    fprintf(stderr, "Error: unable to get iterator\n");
                     exit(1);
                 }
                 while (grib_keys_iterator_next(iter)) {
                     name = grib_keys_iterator_get_name(iter);
-                    /*printf("----- comparing %s\n",name);*/
 
                     if (blocklisted(name))
                         continue;
@@ -893,49 +629,25 @@ static int compare_handles(grib_handle* h1, grib_handle* h2, grib_runtime_option
     else {
         const void *msg1 = NULL, *msg2 = NULL;
         size_t size1 = 0, size2 = 0;
-        int memcmp_ret = 0;
         /* int ii=0; */
         GRIB_CHECK_NOLINE(grib_get_message(h1, &msg1, &size1), 0);
         GRIB_CHECK_NOLINE(grib_get_message(h2, &msg2, &size2), 0);
-        if (size1 == size2 && !(memcmp_ret = memcmp(msg1, msg2, size1))) {
+        if ( size1 == size2 && (0 == memcmp(msg1, msg2, size1)) ) {
             return 0;
         }
-#if 0
-        else {
-            int lcount=count,ii;
-            if (options->current_infile) lcount=options->current_infile->filter_handle_count;
-            if (size1 != size2) {
-                printf("#%d different size: %d!=%d\n",lcount,(int)size1,(int)size2);
-            }
-            if (memcmp_ret) {
-                unsigned char *m1=(unsigned char*)msg1;
-                unsigned char *m2=(unsigned char*)msg2;
-                printf("=== list of different bytes for message %d\n",lcount);
-                for (ii=0;ii<size1;ii++) {
-                    if (memcmp(m1,m2,1)) {
-                        printf("  %d 0x%.2X != 0x%.2X\n",ii,*m1,*m2);
-                    }
-                    m1++; m2++;
-                }
-            }
-            return err;
-        }
-#endif
-
 
         if (listFromCommandLine) {
             for (i = 0; i < options->compare_count; i++) {
                 if (blocklisted(name))
                     continue;
-                if (options->compare[i].type == GRIB_NAMESPACE) {
+                if (options->compare[i].type == CODES_NAMESPACE) {
                     iter = grib_keys_iterator_new(h1, 0, options->compare[i].name);
                     if (!iter) {
-                        printf("ERROR: unable to get iterator for %s\n", options->compare[i].name);
+                        fprintf(stderr, "Error: unable to get iterator for %s\n", options->compare[i].name);
                         exit(1);
                     }
                     while (grib_keys_iterator_next(iter)) {
                         name = grib_keys_iterator_get_name(iter);
-                        /*printf("----- comparing %s\n",name);*/
 
                         if (blocklisted(name))
                             continue;
